############### adaptive elastic net regression within caret workflow ##########################

# set working directory
setwd("D:\\...")

################### adaptive elastic net regression ##########################
# load required packages
if(!require(tidyverse)){
  install.packages("tidyverse")
  library(tidyverse)
}
if(!require(glmnet)){
  install.packages("glmnet")
  library(glmnet)
}
if(!require(msaenet)){
  install.packages("msaenet")
  library(msaenet)
}
if(!require(caret)){
  install.packages("caret")
  library(caret)
}

# read in EMA data
EMA <- read.xlsx("Supplementary data_EMA assessment.xlsx")

# establish Y variable: ratio of SCF to all answered trials
Yratio <- ((EMA$ConEnact)/(EMA$All - EMA$Miss)) # SCF per all answered questionnaires


# wrangle data into befitting format
Y <- Yratio

# decide which outcome
Ydata <- Y

########### X ###############
# read EEG data ERN
X <- readMat("Supplementary data_EEG_R_err_incomp_BL_-400_-200_smoothing_5.mat")
str(X)

# wrangle data into befitting format
X <- as.matrix(X)        # ansteuern X: X[[1]][el,vp,tp]  --> [63x131x750]
X <- as.array(X[[1]])    # ansteuern X: X[el,vp,tp]  --> [63x131x750]
dim(X)

# wrangle data into befitting format
Xdata <- array(0, dim = c(131, 4025)) # empty matrix - read in 175 tp of epoch -50 ms to +300 ms for 23 electrodes
# which electrodes to use in cluster (indices)
elcluster <- c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,26,27,28,29)  # bigger central cluster 23 electrodes


# for loop for building X matrix
counter = 0
for (el in elcluster) {
  counter = counter+1

  for (tp in 226:400) {    # read in 175 tp of epoch -50 ms to +300 ms
    for (vp in 1:dim(X)[2]) {

      Xdata[vp,(((counter-1)*175)+(tp-225))] <- X[el,vp,tp] # read in 175 tp of epoch -50 ms to +300 ms
      
    } } }

#Creating a new Matrix using rbind()  
Data <- cbind(Xdata, Ydata)  
#Printing that Matrix  
Data <- as.data.frame(Data)
Data$Ydata


#################### build model ####################

# read in custom function for glmnet that does elastic net regularization
source("glmnet2_enetNoLasso.r")

# set train control settings for enet/aenet
cv = trainControl(method = "repeatedcv", number = 5, repeats = 10, 
                  returnResamp = "all")
# set train Control for bootstrap
bootstrap <- trainControl(method = "boot632", number = 1000, returnResamp = "all",                         
                          summaryFunction = defaultSummary, 
                          savePredictions = TRUE)
scale <- 1

######################################
# aenet in caret - in 2 steps
######################################

# first step without penalty factors
cat("Starting step 1: elastic net regression ...\n")
  
set.seed(28)
enet.cv <- train(Ydata ~ ., data = Data, method = glmnet2.ENL, family = "poisson", 
             trControl = cv, tuneLength = 5)
  
cat("Finished tuning enet model...\n") 
  
best.alpha.enet <- enet.cv$bestTune$alpha
best.lambda.enet <- enet.cv$bestTune$lambda

cat("Calculating penalty factors...\n")
  
bhat <- as.matrix(enet.cv$finalModel$beta)
if (all(bhat == 0)) bhat <- rep(.Machine$double.eps * 2, length(bhat))
adpen <- (pmax(abs(bhat), .Machine$double.eps))^(-scale)

# second step with calculated penalty factors
cat("Starting step 2: adaptive elastic net regression ...\n")

set.seed(28)
aenet.cv <- train(Ydata ~ ., data = Data, method = glmnet2.ENL, family = "poisson",
                      trControl = cv, tuneLength = 5, penalty.factor = adpen)

cat("Finished tuning aenet model...\n") 

best.alpha.aenet <- aenet.cv$bestTune$alpha
best.lambda.aenet <- aenet.cv$bestTune$lambda

##############################################
# bootstrap aenet model
##############################################

set.seed(28)
FinalEL<-train(Ydata ~ ., data = Data, method = glmnet2.ENL, family = "poisson",trControl = bootstrap, metric = "RMSE",                 
               tuneGrid = expand.grid(alpha = best.alpha.aenet, lambda = best.lambda.aenet), penalty.factor = adpen)
# get important variables
varimp_FinalEL <- varImp(FinalEL)
plot(varimp_FinalEL, main="Variable Importance with elnet", top = 20)


##### save results ####
save(enet.cv, aenet.cv, FinalEL, varimp_FinalEL, adpen, file = "aenet__2times10repeats_1000bootstrap.RData")




